/* ==========================================================
 * Program: 24-HorseshoeP1P3.cpp
 * ==========================================================
 * The program checks the covering relations between some 
 * h-sets related to the connecting orbit between P1 and P3
 * From the program 06:
 	P1 ~ 1.098956671;	dom. eigenvalue approx. 264
	P3 ~ 1.712042516;	dom. eigenvalue approx. 5
 * ==========================================================*/ 

#include <iostream>
using namespace std;

#include "capd/capdlib.h"
using namespace capd;
using namespace capd::alglib;
using namespace capd::matrixAlgorithms;
#include "capd/covrel/HSet2D.h"
#include "capd/dynsys/DynSysMap.h"

#include "utils.h"
typedef capd::covrel::HSet2D<DMatrix, IMatrix> HSet2D;


int main(int argc, char* argv[])
{  	
	cout.precision(8);
   	
   	mojUklad hyperion;					// structure storing Poincare map of Hyperion rotation system, see utils
   	
	//---------------------------------stationary points------------------------------------------------------	
	interval z3=1.711370591161749;
	interval z1=1.099105177037064;				// approximately
	IMatrix Mv3(3,3), Mv1(3,3);
	IVector p3=hyperion.pStNaPi2(z3,Mv3);	// rigorously, small intervals containing P1 and P3
	IVector p1=hyperion.pStNaPi2(z1,Mv1);
	//cout << "Mv1 = " << Mv1 << ", " << "\nMv3 = " << Mv3 << endl;		// eigenvectors' matrices
	
	//---------------------------------base points of h-sets------------------------------------------------------	
	IVector o[5];
	o[0]=p1;
	o[1][0]=1.68635; 		
	o[1][1]=1.21391;
	o[2][0]=1.93186; 
	o[2][1]=1.62049;
	o[3][0]=1.6471; 
	o[3][1]=1.67581;
	o[4]=p3;
	
	//------------------------------------the base cubes of h-sets ----------------------------------------------------
	double r = 1*pow(10,-3);
	IVector b[5];
	b[0][0]=interval(-0.65*r,0.65*r);					
	b[0][1]=interval(-0.65*r,0.65*r);
	b[1][0]=interval(-3*r,3*r);				
	b[1][1]=interval(-10*r,10*r);
	b[2][0]=interval(-20*r,20*r);			
	b[2][1]=interval(-8*r,8*r);
	b[3][0]=interval(-10*r,10*r);			
	b[3][1]=interval(-15*r,15*r);
	b[4][0]=interval(-25*r,25*r);			
	b[4][1]=interval(-25*r,25*r);
   	//--------------------------------- covering check (some cases with 10x10 or 20x20 griding)------------------------
	try{
		if(hyperion.covers(b[0],Mv1,o[0],b[0],Mv1,o[0],20,20))
		{			
			cout << "Covering! N0 =P=> N0" << endl;	
			cout << "----------------------------------------------------" << endl;		
		}
		if(hyperion.covers(b[0],Mv1,o[0],b[1],Mv1,o[1]))
		{			
			cout << "Covering! N0 =P=> N1" << endl;	
			cout << "----------------------------------------------------" << endl;		
		}
		if(hyperion.covers(b[1],Mv1,o[1],b[2],Mv1,o[2],20,20))
		{			
			cout << "Covering! N1 =P=> N2" << endl;	
			cout << "----------------------------------------------------" << endl;		
		}
		if(hyperion.covers(b[2],Mv1,o[2],b[3],Mv3,o[3],20,20))
		{			
			cout << "Covering! N2 =P=> N3" << endl;	
			cout << "----------------------------------------------------" << endl;		
		} 
		if(hyperion.covers(b[3],Mv3,o[3],b[4],Mv3,o[4],10,10))
		{			
			cout << "Covering! N3 =P=> N4" << endl;	
			cout << "----------------------------------------------------" << endl;		
		} 
		if(hyperion.covers(b[4],Mv3,o[4],b[4],Mv3,o[4],10,10))
		{			
			cout << "Covering! N4 =P=> N4" << endl;	
			cout << "----------------------------------------------------" << endl;		
		}	
		}
	catch(exception& e)
		{
    		cout << "\n\nException caught: "<< e.what() << endl;
		}
  return 0;
} 

/* OUTPUT: 
Covering! N0 =P=> N0
----------------------------------------------------
Covering! N0 =P=> N1
----------------------------------------------------
Covering! N1 =P=> N2
----------------------------------------------------
Covering! N2 =P=> N3
----------------------------------------------------
Covering! N3 =P=> N4
----------------------------------------------------
Covering! N4 =P=> N4
----------------------------------------------------


*/
