/* ==========================================================
 * Program: 25c-HorseshoeP3P3.cpp
 * ==========================================================
 * The program checks the covering relations between some 
 * h-sets related to the connecting orbit between P3 and P3
 * From the program 06:
	P3 ~ 1.712042516;	dom. eigenvalue approx. 5
 * ==========================================================*/
 
#include <iostream>
using namespace std;

#include "capd/capdlib.h"
using namespace capd;
using namespace capd::alglib;
using namespace capd::matrixAlgorithms;
#include "capd/covrel/HSet2D.h"
#include "capd/dynsys/DynSysMap.h"

#include "utils.h"
typedef capd::covrel::HSet2D<DMatrix, IMatrix> HSet2D;


int main(int argc, char* argv[])
{  	
	cout.precision(8);
	
	//------------------------------Hyperion Poincare Map---------------------------------------------
	mojUklad hyperion;					// structure storing Poincare map of Hyperion rotation system, see utils
	mojUklad hyperion2(2);				// structure storing P^2
	//---------------------------------stationary points------------------------------------------------------	
	interval z3=1.711370591161749;			// approximately
	IMatrix Mv3(3,3);
	IVector p3=hyperion.pStNaPi2(z3,Mv3);	// rigorously, small interval containing P3
	
	//--------------------------------- adjusted matrices ------------------------------------------------------	
	IMatrix m2(3,3);
	m2[0][0]=interval(0.734429);
	m2[0][1]=m2[0][0];
	m2[1][0]=interval(0.678686);
	m2[1][1]=-m2[1][0];
	m2[2][2]=interval(1);
	IMatrix m4=m2;
	m4[0][0]=interval(0.866025);
	m4[1][1]=m4[0][0];
	m4[1][0]=interval(0.5);
	m4[0][1]=-m4[1][0];
	
	//---------------------------------base points of h-sets------------------------------------------------------	
	IVector o[5];
	o[0]=p3;
	o[1][0]= 1.51877;
	o[1][1]= 1.68699;
	o[2][0]= 1.32082;
	o[2][1]= 1.62293;
	o[3][0]= 1.82077;
	o[3][1]= 1.62293;
	o[4][0]= 1.62282;
	o[4][1]= 1.68699;	
	
	//------------------------------------the base cubes of h-sets ----------------------------------------------------
	double r = 1*pow(10,-3);
	IVector b[5];
	b[0][0]=interval(-20*r,20*r);
	b[0][1]=interval(-30*r,30*r);
	b[1][0]=interval(-6*r,6*r);
	b[1][1]=interval(-30*r,30*r);
	b[2][0]=interval(-0.5*r,0.5*r); 
	b[2][1]=interval(-30*r,30*r);
	b[3][0]=interval(-15*r,15*r);	
	b[3][1]=interval(-15*r,15*r);	
	b[4][0]=interval(-20*r,20*r);
	b[4][1]=interval(-20*r,20*r);	
	
   	//--------------------------------- covering check (all with 10x10 griding)------------------------
	try{
		if(hyperion.covers(b[0],Mv3,o[0],b[0],Mv3,o[0],10,10))
		{			
			cout << "Covering! N0 =P=> N0" << endl;	
			cout << "----------------------------------------------------" << endl;		
		}
		if(hyperion.covers(b[0],Mv3,o[0],b[1],Mv3,o[1],10,10))
		{			
			cout << "Covering! N0 =P=> N1" << endl;	
			cout << "----------------------------------------------------" << endl;		
		}	
		if(hyperion.covers(b[1],Mv3,o[1],b[2],m2,o[2],10,10))
		{			
			cout << "Covering! N1 =P=> N2" << endl;	
			cout << "----------------------------------------------------" << endl;		
		}	
		if(hyperion2.covers(b[2],m2,o[2],b[3],m4,o[3],10,10))
		{			
			cout << "Covering! N2 =P^2=> N3" << endl;	
			cout << "----------------------------------------------------" << endl;		
		}
		if(hyperion.covers(b[3],m4,o[3],b[4],Mv3,o[4],10,10))
		{			
			cout << "Covering! N3 =P=> N4" << endl;	
			cout << "----------------------------------------------------" << endl;		
		}
		if(hyperion.covers(b[4],Mv3,o[4],b[0],Mv3,o[0],10,10))
		{			
			cout << "Covering! N4 =P=> N0" << endl;	
			cout << "----------------------------------------------------" << endl;		
		}	
		if(hyperion.covers(b[4],Mv3,o[4],b[1],Mv3,o[1],10,10))
		{			
			cout << "Covering! N4 =P=> N1" << endl;	
			cout << "----------------------------------------------------" << endl;		
		}	
		}
	catch(exception& e)
		{
    		cout << "\n\nException caught: "<< e.what() << endl;
		}
  return 0;
} 

/* OUTPUT: 
Covering! N0 =P=> N0
----------------------------------------------------
Covering! N0 =P=> N1
----------------------------------------------------
Covering! N1 =P=> N2
----------------------------------------------------
Covering! N2 =P^2=> N3
----------------------------------------------------
Covering! N3 =P=> N4
----------------------------------------------------
Covering! N4 =P=> N0
----------------------------------------------------
Covering! N4 =P=> N1
----------------------------------------------------

*/
